import { HardhatRuntimeEnvironment } from "hardhat/types";
import { DeployFunction } from "hardhat-deploy/types";
import { updateArray } from "../scripts/utils"


const deployNFTClaim: DeployFunction = 
  async function (hre: HardhatRuntimeEnvironment) {
    const provider = hre.ethers.provider;
    const from = await (await provider.getSigner()).getAddress();
    const config = require(`../config/config_${hre.network.name}`);
    const { mallFeeAddress, paymentTokens, verifier } = config.market
    const ret = await hre.deployments.deploy("GameItemMall", {
      from,
      args: [paymentTokens[0], mallFeeAddress, verifier, 3600],
      log: true,
    });
    console.log("==GameItemMall addr=", ret.address);
    updateArray({
      name: "GameItemMall",
      type: "logic",
      json: "assets/contracts/GameItemMall.json",
      address: ret.address,
      network: hre.network.name,
    });
    await hre.run("verify:verify", {
      address: ret.address,
      constructorArguments: [paymentTokens[0], mallFeeAddress, verifier, 3600],
    });
  };

  deployNFTClaim.tags = ["GameItemMall"];

export default deployNFTClaim;
