import { HardhatRuntimeEnvironment } from "hardhat/types";
import { DeployFunction } from "hardhat-deploy/types";
import { updateArray } from "../scripts/utils"


const deployNFTClaim: DeployFunction = 
  async function (hre: HardhatRuntimeEnvironment) {
    const provider = hre.ethers.provider;
    const from = await (await provider.getSigner()).getAddress();
    const config = require(`../config/config_${hre.network.name}`);
    const { mallFeeAddress, paymentTokens, verifier } = config.market
    const ret = await hre.deployments.deploy("TokenClaim", {
      from,
      args: [mallFeeAddress, paymentTokens[0], verifier, 3600],
      log: true,
    });
    console.log("==TokenClaim addr=", ret.address);
    updateArray({
      name: "TokenClaim",
      type: "logic",
      json: "assets/contracts/TokenClaim.json",
      address: ret.address,
      network: hre.network.name,
    });
    // verify the contract
    await hre.run("verify:verify", {
      address: ret.address,
      constructorArguments: [mallFeeAddress, paymentTokens[0], verifier, 3600],
    });
  };

  deployNFTClaim.tags = ["TokenClaim"];

export default deployNFTClaim;
